//---------------------------------------------------------------------------

#ifdef _WIN32
 #include <windows.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <GL/glew.h>
#include <GL/glut.h>

//#include "texture.cpp"


//---------------------------------------------------------------------------

GLuint gWinId = 0;
GLuint gResx = 0;
GLuint gResy = 0;


GLhandleARB gFragment = 0;

extern GLuint loadTGA(const char* fname);
GLuint texID=0;
//fragment shader

//standardny postup je mat ulozeny shader na disku a nacitat ho
//kvoli jdnoduchosti je shader ako string

const char* fragmentSrc = {"							    \n\
                                                            \n\
uniform sampler2D texCol;                                   \n\
uniform vec4 col;										    \n\
void main()												    \n\
{														    \n\
	gl_FragData[0] = col*texture2D(texCol, 2.0*gl_TexCoord[0].xy);	\n\
}														    \n\
                                                            \n\
"};



//---------------------------------------------------------------------------

void PrintShaderLog(const GLhandleARB obj_in)
{
	int infoLogLength = 0;
	int charsWritten  = 0;
	char *infoLog;

	glGetObjectParameterivARB(obj_in, GL_OBJECT_INFO_LOG_LENGTH_ARB, &infoLogLength);

	if (infoLogLength > 1)
	{
		infoLog = (char *)malloc(infoLogLength);
		if(!infoLog)
		{
			fprintf(stderr,"Allocation memory failed.\n");
			return;
		}
		glGetInfoLogARB(obj_in, infoLogLength, &charsWritten, infoLog);
		fprintf(stderr, "shaderlog (%d): \"%s\"\n",infoLogLength,infoLog);
		free(infoLog);
	}
}


bool CreateShader()
{
	//create and cmpile shader OBJECT
	GLhandleARB shaderObj = glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);
	glShaderSourceARB(shaderObj, 1, &fragmentSrc, NULL);
	glCompileShaderARB(shaderObj);
	//print shader log (errors)
	PrintShaderLog(shaderObj);

	//create shader program
	gFragment = glCreateProgramObjectARB();
	//attach shader object to shader program
	glAttachObjectARB(gFragment, shaderObj);
	//link shader program
	glLinkProgramARB(gFragment);
	//print shader log (errors)
	PrintShaderLog(gFragment);

	//set shader name parameters
	GLint shColor = glGetUniformLocation(gFragment, "col");
    GLint shColorMapID=glGetUniformLocation(gFragment,"colorMap");

	//enable shader
	glUseProgramObjectARB(gFragment);

	//set shader parameters values
	float col[4]={1.0,0.0,0.0,1.0};
	glUniform4f(shColor, 1.0, 0.5, 0.2, 1.0);
   glUniform1i(shColorMapID,0);

	//disable shader
//	glUseProgramObjectARB(0);

	return true;
}



void Redraw()
{
    glClear(GL_COLOR_BUFFER_BIT);

	GLfloat d=5.0;
	glColor4f(1.0,1.0,1.0,1.0);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D,texID);
	glBegin(GL_QUADS);
		glTexCoord2f(0.0, 0.0);
		glVertex2f(0.0+d, 0.0+d);
		glTexCoord2f(0.0, 1.0);
		glVertex2f(0.0+d, gResy-d);
		glTexCoord2f(1.0, 1.0);
		glVertex2f(gResx-d, gResy-d);
		glTexCoord2f(1.0, 0.0);
		glVertex2f(gResx-d, 0.0+d);
	glEnd();
    glDisable(GL_TEXTURE_2D);
    glutSwapBuffers();
#ifdef _WIN32
    Sleep(50);
#else
    #ifdef UNIX
        usleep(50);
    #endif
#endif
}


void ReSize(int width_in, int height_in)
{
    glViewport(0, 0, width_in, height_in);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(0, width_in, 0, height_in, -1, 1);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	gResx = width_in;
	gResy = height_in;

	Redraw();
}


void MouseMove(int x_in, int y_in)
{
}


void MousePress(int button_in, int state_in, int x_in, int y_in)
{
}


void KeyPressed(unsigned char key_in, int x_in, int y_in)
{
    switch(key_in)
	{
		case 27:
			{
                glutDestroyWindow(gWinId);
			    exit(0);
			}
			break;
	}
}


int main(int argc, char **argv)
{
    //set glut window settings
    glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH );
	glutInitWindowSize(640, 480);
	glutInitWindowPosition(100, 100);
    //create glut window
    gWinId = glutCreateWindow("glsl_example");

    //set glut contorling function
	glutDisplayFunc(&Redraw);
	glutIdleFunc(&Redraw);
	glutReshapeFunc(&ReSize);
	glutKeyboardFunc(&KeyPressed);
	glutMouseFunc(&MousePress);
	glutMotionFunc(&MouseMove);

	if (GLEW_OK != glewInit())
    {
		fprintf(stderr,"Error glew initialization failed.\n");
    }

    texID=loadTGA("lena.tga");
	CreateShader();


    //enable main loop
	glutMainLoop();

	return 0;
}

