#ifdef _WIN32
 #include <windows.h>
#endif

#include <GL/gl.h>
#include <stdlib.h>
#include <stdio.h>

typedef unsigned char byte;

#pragma pack(push)
#pragma pack(1)

typedef struct
{
    byte  identsize;          // size of ID field that follows 18 byte header (0 usually)
    byte  colourmaptype;      // type of colour map 0=none, 1=has palette
    byte  imagetype;          // type of image 0=none,1=indexed,2=rgb,3=grey,+8=rle packed

    short colourmapstart;     // first colour map entry in palette
    short colourmaplength;    // number of colours in palette
    byte  colourmapbits;      // number of bits per palette entry 15,16,24,32

    short xstart;             // image x origin
    short ystart;             // image y origin
    short width;              // image width in pixels
    short height;             // image height in pixels
    byte  bpp;               // image bits per pixel 8,16,24,32
    byte  descriptor;         // image descriptor bits (vh flip bits)

    // pixel data follows header

} TGA_HEADER;

#pragma pack(pop)

GLuint loadTGA(const char* fname)
{
	FILE* f=fopen(fname,"rb");
	if (!f)
	{
	    fprintf(stderr,"File not open\n");
        return 0;
	}
	TGA_HEADER head;
	fread(&head,1,sizeof(TGA_HEADER),f);
	GLvoid* pixels=0;
	GLuint texID=0;
	if (head.imagetype==2) // RGB not packed
	{
		if (head.bpp==24)
		{
			pixels=(unsigned char*)malloc(head.width*head.height*3);
			fread(pixels,head.height,head.width*(head.bpp>>3),f);
			glGenTextures(1,&texID);
			glBindTexture(GL_TEXTURE_2D,texID);
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);	// Linear Filtering
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);	// Linear Filtering

			if ((head.descriptor & 0x30) == 0) // dolu hlavou, BGR
				glTexImage2D(GL_TEXTURE_2D,0,GL_RGB8,head.width,head.height,0,GL_BGR_EXT,GL_UNSIGNED_BYTE,pixels);

		}
	}
	free(pixels);
	fclose(f);
	return texID;
}



